@@ -7,6 +7,7 @@ from .extract_sparse_array import extract_sparse_array
 from .create_dask_array import create_dask_array
 from .chunk_shape import chunk_shape
 from .is_sparse import is_sparse
+from .is_masked import is_masked
 
 __author__ = "ltla"
 __copyright__ = "ltla"
@@ -94,3 +95,9 @@ def chunk_shape_Cast(x: Cast):
 def is_sparse_Cast(x: Cast):
     """See :py:meth:`~delayedarray.is_sparse.is_sparse`."""
     return is_sparse(x._seed)
+
+
+@is_masked.register
+def is_masked_Cast(x: Cast):
+    """See :py:meth:`~delayedarray.is_masked.is_masked`."""
+    return is_masked(x._seed)
