@@ -1,13 +1,16 @@
 from typing import Callable, Optional, Tuple, Sequence
-from numpy import concatenate, dtype, ndarray
+import numpy
 
 from .DelayedOp import DelayedOp
 from ._subset import _spawn_indices
+from ._mask import _concatenate_unmasked_ndarrays, _concatenate_maybe_masked_ndarrays
 from .extract_dense_array import extract_dense_array, _sanitize_to_fortran
+from .SparseNdarray import _concatenate_SparseNdarrays
 from .extract_sparse_array import extract_sparse_array
 from .create_dask_array import create_dask_array
 from .chunk_shape import chunk_shape
 from .is_sparse import is_sparse
+from .is_masked import is_masked
 
 __author__ = "ltla"
 __copyright__ = "ltla"
@@ -22,17 +25,19 @@ class Combine(DelayedOp):
 
     This class is intended for developers to construct new :py:class:`~delayedarray.DelayedArray.DelayedArray`
     instances. In general, end users should not be interacting with ``Combine`` objects directly.
-
-    Attributes:
-        seeds (list):
-            List of objects that satisfy the seed contract,
-            see :py:class:`~delayedarray.DelayedArray.DelayedArray` for details.
-
-        along (int):
-            Dimension along which the seeds are to be combined.
     """
 
     def __init__(self, seeds: list, along: int):
+        """
+        Args:
+            seeds:
+                List of objects that satisfy the seed contract,
+                see :py:class:`~delayedarray.DelayedArray.DelayedArray` for details.
+
+            along:
+                Dimension along which the seeds are to be combined.
+        """
+
         self._seeds = seeds
         if len(seeds) == 0:
             raise ValueError("expected at least one object in 'seeds'")
@@ -56,49 +61,47 @@ class Combine(DelayedOp):
         # Guessing the dtype.
         to_combine = []
         for i in range(len(seeds)):
-            to_combine.append(ndarray((0,), dtype=seeds[i].dtype))
-        self._dtype = concatenate((*to_combine,)).dtype
+            to_combine.append(numpy.ndarray((0,), dtype=seeds[i].dtype))
+        self._dtype = _concatenate_unmasked_ndarrays(to_combine, axis=0).dtype
+
+        self._is_masked = any(is_masked(y) for y in self._seeds)
 
     @property
     def shape(self) -> Tuple[int, ...]:
-        """Shape of the ``Combine`` object.
-
+        """
         Returns:
-            Tuple[int, ...]: Tuple of integers specifying the extent of each dimension of the ``Combine`` object,
-            (i.e., after seeds were combined along the ``along`` dimension).
+            Tuple of integers specifying the extent of each dimension of the
+            object after seeds were combined along the specified dimension.
         """
         return self._shape
 
     @property
-    def dtype(self) -> dtype:
-        """Type of the ``Combine`` object. This may or may not be the same as those in ``seeds``, depending on casting
-        rules.
-
+    def dtype(self) -> numpy.dtype:
+        """
         Returns:
-            dtype: NumPy type for the ``Combine`` contents.
+            NumPy type for the combined data.  This may or may not be
+            the same as those in ``seeds``, depending on casting rules.
         """
         return self._dtype
 
     @property
     def seeds(self) -> list:
-        """Get the list of underlying seed objects.
-
+        """
         Returns:
-            list: List of seeds.
+            List of seed objects to be combined.
         """
         return self._seeds
 
     @property
     def along(self) -> int:
-        """Dimension along which the seeds are combined.
-
+        """
         Returns:
-            int: Dimension to combine along.
+            Dimension along which the seeds are combined.
         """
         return self._along
 
 
-def _extract_array(x: Combine, subset: Optional[Tuple[Sequence[int]]], f: Callable):
+def _extract_subarrays(x: Combine, subset: Optional[Tuple[Sequence[int], ...]], f: Callable):
     if subset is None:
         subset = _spawn_indices(x.shape)
 
@@ -124,20 +127,22 @@ def _extract_array(x: Combine, subset: Optional[Tuple[Sequence[int]]], f: Callab
             flexargs[x._along] = fragmented[i]
             extracted.append(f(s, (*flexargs,)))
 
-    return concatenate((*extracted,), axis=x._along)
+    return extracted
 
 
 @extract_dense_array.register
-def extract_dense_array_Combine(x: Combine, subset: Optional[Tuple[Sequence[int]]] = None):
+def extract_dense_array_Combine(x: Combine, subset: Optional[Tuple[Sequence[int], ...]] = None):
     """See :py:meth:`~delayedarray.extract_dense_array.extract_dense_array`."""
-    out = _extract_array(x, subset, extract_dense_array)
-    return _sanitize_to_fortran(out)
+    fragments = _extract_subarrays(x, subset, extract_dense_array)
+    combined = _concatenate_maybe_masked_ndarrays(fragments, axis=x._along, masked=x._is_masked)
+    return _sanitize_to_fortran(combined)
 
 
 @extract_sparse_array.register
-def extract_sparse_array_Combine(x: Combine, subset: Optional[Tuple[Sequence[int]]] = None):
+def extract_sparse_array_Combine(x: Combine, subset: Optional[Tuple[Sequence[int], ...]] = None):
     """See :py:meth:`~delayedarray.extract_sparse_array.extract_sparse_array`."""
-    return _extract_array(x, subset, extract_sparse_array)
+    fragments = _extract_subarrays(x, subset, extract_sparse_array)
+    return _concatenate_SparseNdarrays(fragments, along=x._along)
 
 
 @create_dask_array.register
@@ -146,7 +151,7 @@ def create_dask_array_Combine(x: Combine):
     extracted = []
     for s in x._seeds:
         extracted.append(create_dask_array(s))
-    return concatenate((*extracted,), axis=x._along)
+    return numpy.concatenate((*extracted,), axis=x._along)
 
 
 @chunk_shape.register
@@ -175,3 +180,9 @@ def is_sparse_Combine(x: Combine):
         if not is_sparse(s):
             return False
     return len(x._seeds) > 0
+
+
+@is_masked.register
+def is_masked_Combine(x: Combine):
+    """See :py:meth:`~delayedarray.is_masked.is_masked`."""
+    return x._is_masked
