@@ -11,6 +11,7 @@ from .extract_sparse_array import extract_sparse_array
 from .create_dask_array import create_dask_array
 from .chunk_shape import chunk_shape
 from .is_sparse import is_sparse
+from .is_masked import is_masked
 
 __author__ = "ltla"
 __copyright__ = "ltla"
@@ -29,35 +30,38 @@ class UnaryIsometricOpWithArgs(DelayedOp):
 
     This class is intended for developers to construct new :py:class:`~delayedarray.DelayedArray.DelayedArray`
     instances. In general, end-users should not be interacting with ``UnaryIsometricOpWithArgs`` objects directly.
-
-    Attributes:
-        seed:
-            Any object satisfying the seed contract,
-            see :py:meth:`~delayedarray.DelayedArray.DelayedArray` for details.
-
-        value (Union[float, ndarray]):
-            A scalar or NumPy array with which to perform an operation on the ``seed``.
-
-            If scalar, the operation is applied element-wise to all entries of ``seed``.
-
-            If a 1-dimensional NumPy array, the operation is broadcast along the last dimension of ``seed``.
-
-            If an n-dimensional NumPy array, the number of dimensions should be equal to the dmensionality of ``seed``.
-            All dimensions should be of extent 1, except for exactly one dimension that should have extent equal to the
-            corresponding dimension of ``seed``. The operation is then broadcast along that dimension.
-
-        operation (str):
-            String specifying the operation.
-
-        right (bool, optional):
-            Whether ``value`` is to the right of ``seed`` in the operation.
-            If False, ``value`` is put to the left of ``seed``.
-            Ignored for commutative operations in ``op``.
     """
 
     def __init__(self, seed, value: Union[float, ndarray], operation: ISOMETRIC_OP_WITH_ARGS, right: bool = True):
+        """
+        Args:
+            seed:
+                Any object satisfying the seed contract, see
+                :py:meth:`~delayedarray.DelayedArray.DelayedArray` for details.
+
+            value:
+                A scalar or NumPy array with which to perform an operation on the ``seed``.
+
+                If scalar, the operation is applied element-wise to all entries of ``seed``.
+
+                If a 1-dimensional NumPy array, the operation is broadcast along the last dimension of ``seed``.
+
+                If an n-dimensional NumPy array, the number of dimensions should be equal to the dmensionality of ``seed``.
+                All dimensions should be of extent 1, except for exactly one dimension that should have extent equal to the
+                corresponding dimension of ``seed``. The operation is then broadcast along that dimension.
+
+            operation:
+                String specifying the operation.
+
+            right:
+                Whether ``value`` is to the right of ``seed`` in the operation.
+                If False, ``value`` is put to the left of ``seed``. Ignored
+                for commutative operations in ``op``.
+        """
+
         along = _infer_along_with_args(seed.shape, value)
         if along is None and isinstance(value, ndarray):
+            ndim = len(value.shape)
             value = value[(*([0] * ndim),)]
 
         with warnings.catch_warnings():  # silence warnings from divide by zero.
@@ -79,78 +83,71 @@ class UnaryIsometricOpWithArgs(DelayedOp):
 
     @property
     def shape(self) -> Tuple[int, ...]:
-        """Shape of the ``UnaryIsometricOpWithArgs`` object. As the name of the class suggests, this is the same as the
-        ``seed`` array.
-
+        """
         Returns:
-            Tuple[int, ...]: Tuple of integers specifying the extent of each dimension of the
-            ``UnaryIsometricOpWithArgs`` object.
+            Tuple of integers specifying the extent of each dimension of this
+            object. This should be the same as ``seed``.
         """
         return self._seed.shape
 
     @property
     def dtype(self) -> numpy.dtype:
-        """Type of the ``UnaryIsometricOpWithArgs`` object. This may or may not be the same as the ``seed`` array,
-        depending on how NumPy does the casting for the requested operation.
-
+        """
         Returns:
-            dtype: NumPy type for the ``UnaryIsometricOpWithArgs`` contents.
+            NumPy type for the data after the operation was applied. This may
+            or may not be the same as the ``seed`` array, depending on how
+            NumPy does the casting for the requested operation.
         """
         return self._dtype
 
     @property
     def seed(self):
-        """Get the underlying object satisfying the seed contract.
-
+        """
         Returns:
             The seed object.
         """
         return self._seed
 
     @property
-    def operation(self) -> str:
-        """Get the name of the operation.
-
+    def operation(self) -> ISOMETRIC_OP_WITH_ARGS:
+        """
         Returns:
-            str: Name of the operation.
+            Name of the operation.
         """
         return self._op
 
     @property
     def value(self) -> Union[float, ndarray]:
-        """Get the other operand used in the operation.
-
+        """
         Returns:
-            Union[float, ndarray]: The other operand.
-            This can be a numeric scalar or a NumPy array.
+            The other operand used in the operation.
         """
         return self._value
 
     @property
     def right(self) -> bool:
-        """Is the :py:attr:`~value` applied to the right of the seed?
-
+        """
         Returns:
-            bool: Whether to apply the operation to the right of the seed.
+            Whether the operation was applied to the right of the seed.
         """
         return self._right
 
     @property
     def along(self) -> Union[int, None]:
-        """If :py:attr:`~value` is an array, this specifies the dimension of :py:attr:``~seed`` along which the array
-        values are broadcast.
-
+        """
         Returns:
-            Union[int, None]: Broadcasting dimension, or None if ``value`` is a scalar.
+            The dimension of :py:attr:``~seed`` along which the array values
+            are broadcast, for an array :py:attr:`~value`. Otherwise None, if
+            ``value`` is a scalar.
         """
         return self._along
 
 
-def _extract_array(x: UnaryIsometricOpWithArgs, subset: Optional[Tuple[Sequence[int]]], f: Callable): 
+def _extract_array(x: UnaryIsometricOpWithArgs, subset: Optional[Tuple[Sequence[int], ...]], f: Callable): 
     target = f(x._seed, subset)
 
     subvalue = x._value
-    if isinstance(subvalue, ndarray):
+    if isinstance(subvalue, ndarray) and not subvalue is numpy.ma.masked:
         if subset is None:
             subset = _spawn_indices(x.shape)
         if len(subvalue.shape) == 1:
@@ -168,14 +165,14 @@ def _extract_array(x: UnaryIsometricOpWithArgs, subset: Optional[Tuple[Sequence[
 
 
 @extract_dense_array.register
-def extract_dense_array_UnaryIsometricOpWithArgs(x: UnaryIsometricOpWithArgs, subset: Optional[Tuple[Sequence[int]]] = None):
+def extract_dense_array_UnaryIsometricOpWithArgs(x: UnaryIsometricOpWithArgs, subset: Optional[Tuple[Sequence[int], ...]] = None):
     """See :py:meth:`~delayedarray.extract_dense_array.extract_dense_array`."""
     out = _extract_array(x, subset, extract_dense_array)
     return _sanitize_to_fortran(out)
 
 
 @extract_sparse_array.register
-def extract_sparse_array_UnaryIsometricOpWithArgs(x: UnaryIsometricOpWithArgs, subset: Optional[Tuple[Sequence[int]]] = None):
+def extract_sparse_array_UnaryIsometricOpWithArgs(x: UnaryIsometricOpWithArgs, subset: Optional[Tuple[Sequence[int], ...]] = None):
     """See :py:meth:`~delayedarray.extract_sparse_array.extract_sparse_array`."""
     return _extract_array(x, subset, extract_sparse_array)
 
@@ -201,3 +198,10 @@ def chunk_shape_UnaryIsometricOpWithArgs(x: UnaryIsometricOpWithArgs):
 def is_sparse_UnaryIsometricOpWithArgs(x: UnaryIsometricOpWithArgs):
     """See :py:meth:`~delayedarray.is_sparse.is_sparse`."""
     return x._sparse
+
+
+@is_masked.register
+def is_masked_UnaryIsometricOpWithArgs(x: UnaryIsometricOpWithArgs):
+    """See :py:meth:`~delayedarray.is_masked.is_masked`."""
+    return is_masked(x._seed) or numpy.ma.isMaskedArray(x._value) or x._value is numpy.ma.masked
+
